
# q-expansion of newform 350.2.e.e, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 350
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [101, [0, -1]]]
aps_data = [[0, -1], [1, -1], [0, 0], [-1, 3], [6, -6], [4, 0], [0, 0], [0, -2], [0, -3], [-3, 0], [-8, 8], [0, -4], [9, 0], [7, 0], [0, 0], [-6, 6], [6, -6], [0, -5], [5, -5], [-6, 0], [-16, 16], [0, -2], [-3, 0], [0, 15], [-14, 0], [-15, 15], [0, -1], [0, -15], [-11, 11], [-6, 0], [-8, 0], [0, 0], [12, -12], [-10, 0], [0, -15], [4, -4], [-22, 22], [0, -4], [3, 0], [0, 0], [24, -24], [11, 0], [0, 6], [2, -2], [6, 0], [4, -4], [-10, 0], [28, 0], [-12, 12], [0, -14], [0, -12], [12, 0], [-2, 2], [-12, 0], [0, 0], [-21, 21], [-15, 15], [0, -2], [8, -8], [-6, 0], [-4, 4], [-12, 0], [-5, 0], [18, -18], [0, 8], [0, 12], [0, 28], [22, 0], [9, -9], [17, 0], [-6, 6], [0, -24], [35, -35], [0, -4], [-34, 0], [0, -15], [30, -30], [0, 14], [0, -15], [13, -13], [0, 0], [17, 0], [-30, 30], [22, 0], [0, 28], [0, 21], [-9, 0], [0, 32], [18, 0], [13, 0], [0, -15], [-12, 12], [-16, 16], [0, 0], [0, 22], [-21, 0], [0, -21], [18, -18], [0, -28], [0, 25], [19, 0], [-18, 18], [27, -27], [0, -6], [22, -22], [26, -26], [-12, 0], [0, 6], [-12, 12], [-46, 0], [0, 23], [-16, 16], [12, 0], [-14, 14], [14, 0], [3, -3], [-20, 0], [3, -3], [0, -48], [6, 0], [-41, 41], [-8, 0], [0, 12], [9, -9], [0, 22], [-3, 0], [0, 31], [0, 18], [19, 0], [0, -34], [-26, 26], [-39, 0], [0, 4], [28, 0], [0, -42], [50, 0], [-12, 12], [-43, 43], [48, 0], [-21, 21], [-16, 0], [0, 18], [-19, 19], [-15, 0], [-2, 2], [-30, 0], [46, 0], [6, -6], [0, -32], [0, 27], [0, 2], [57, 0], [52, 0], [0, -21], [-25, 25], [18, 0], [0, -14], [0, 21], [28, 0], [6, -6], [0, -3], [-60, 0], [-35, 0], [0, 0], [-6, 6], [39, -39], [28, -28], [14, -14]]
